use std::char;

static INPUT: &str = include_str!("input");

fn main() {
    println!("part 1's result is: {:?}", part1());
    println!("part 2's result is: {:?}", part2());
}

fn part1() -> usize {
    sum_them_up(INPUT.lines())
}

fn sum_them_up(input: impl Iterator<Item = impl AsRef<str>>) -> usize {
    input
        .map(|line| {
            let d1 = line
                .as_ref()
                .chars()
                .find(|c| c.is_ascii_digit())
                .expect("every line has at least one digit");
            let d2 = line
                .as_ref()
                .chars()
                .rev()
                .find(|c| c.is_ascii_digit())
                .expect("every line still has at least one digit");
            format!("{}{}", d1, d2)
                .parse::<usize>()
                .expect("two digits should be parsable as int")
        })
        .sum()
}

static CHECK_MAP: [(&str, char); 9] = [
    ("one", '1'),
    ("two", '2'),
    ("three", '3'),
    ("four", '4'),
    ("five", '5'),
    ("six", '6'),
    ("seven", '7'),
    ("eight", '8'),
    ("nine", '9'),
];

fn extract_digits(line: &str) -> String {
    let mut buf = String::new();
    let mut out = String::new();

    for c in line.chars() {
        if c.is_ascii_digit() {
            out.push(c);
        } else {
            buf.push(c);
            if let Some(digit) = CHECK_MAP
                .into_iter()
                .find_map(|(pat, dig)| Some(dig).filter(|_| buf.ends_with(pat)))
            {
                out.push(digit);
            }
        }
    }

    out
}

fn part2() -> usize {
    sum_them_up(INPUT.lines().map(extract_digits))
}
