use std::cmp::max;

static INPUT: &str = include_str!("input");

fn main() {
    println!("part1: {}", part1());
    println!("part2: {}", part2());
}

struct Game {
    id: usize,
    samples: Vec<RevealedSample>,
}

struct RevealedSample {
    red: Option<usize>,
    green: Option<usize>,
    blue: Option<usize>,
}

impl From<&str> for Game {
    fn from(value: &str) -> Self {
        let (id, rest) = value
            .strip_prefix("Game ")
            .unwrap()
            .split_once(": ")
            .unwrap();
        let id = id.parse().unwrap();

        let samples: Vec<_> = rest.split(';').map(RevealedSample::from).collect();

        Self { id, samples }
    }
}

impl From<&str> for RevealedSample {
    fn from(value: &str) -> Self {
        let mut red = None;
        let mut green = None;
        let mut blue = None;

        value
            .split(',')
            .map(str::trim)
            .map(|s| s.split_once(' ').unwrap())
            .for_each(|(count, color)| match color {
                "red" => red = Some(count.parse().unwrap()),
                "green" => green = Some(count.parse().unwrap()),
                "blue" => blue = Some(count.parse().unwrap()),
                _ => panic!("the disco"),
            });

        Self { red, green, blue }
    }
}

impl RevealedSample {
    fn get_tuple(&self) -> (usize, usize, usize) {
        (
            self.red.unwrap_or(0),
            self.green.unwrap_or(0),
            self.blue.unwrap_or(0),
        )
    }
}

impl Game {
    fn max_seen(&self) -> (usize, usize, usize) {
        self.samples.iter().fold((0, 0, 0), |(r, g, b), sample| {
            let (sr, sg, sb) = sample.get_tuple();
            (max(r, sr), max(g, sg), max(b, sb))
        })
    }

    fn is_possible(&self) -> bool {
        let (r, g, b) = self.max_seen();
        r <= 12 && g <= 13 && b <= 14
    }
}

fn part1() -> usize {
    INPUT
        .lines()
        .map(Game::from)
        .filter(Game::is_possible)
        .map(|game| game.id)
        .sum()
}

fn part2() -> usize {
    INPUT
        .lines()
        .map(Game::from)
        .map(|game| game.max_seen())
        .map(|(r, g, b)| r * g * b)
        .sum()
}
