static INPUT: &str = include_str!("input");

fn main() {
    println!("part 1: {}", part1());
    println!("part 2: {}", part2());
}

#[derive(Clone)]
struct Card {
    id: usize,
    winning: Vec<usize>,
    pulled: Vec<usize>,
}

impl From<&str> for Card {
    fn from(value: &str) -> Self {
        let (id, rest) = value.strip_prefix("Card").unwrap().split_once(':').unwrap();
        let id = id.trim().parse().unwrap();
        let (winning, pulled) = rest
            .split_once('|')
            .map(|(w, p)| {
                (
                    w.trim()
                        .split(' ')
                        .filter(|s| !s.is_empty())
                        .map(|s| s.parse().unwrap())
                        .collect(),
                    p.trim()
                        .split(' ')
                        .filter(|s| !s.is_empty())
                        .map(|s| s.parse().unwrap())
                        .collect(),
                )
            })
            .unwrap();

        Card {
            id,
            winning,
            pulled,
        }
    }
}

impl Card {
    fn value(&self) -> usize {
        match self.win_count() {
            0 => 0,
            1 => 1,
            c => [2].repeat(c - 1).into_iter().product(),
        }
    }

    fn win_count(&self) -> usize {
        self.pulled
            .iter()
            .filter(|n| self.winning.contains(n))
            .count()
    }
}

fn part1() -> usize {
    INPUT.lines().map(Card::from).map(|c| c.value()).sum()
}

struct CardSet {
    cards: Vec<Card>,
    processed: Vec<Card>,
    to_process: Vec<Card>,
}

impl CardSet {
    fn do_pass(&mut self) {
        self.processed.extend(self.to_process.clone());
        let new = self
            .to_process
            .iter()
            .flat_map(|c| (c.id..c.id + c.win_count()).filter_map(|n| self.cards.get(n).cloned()))
            .collect();
        self.to_process = new;
    }

    fn needs_work(&self) -> bool {
        !self.to_process.is_empty()
    }

    fn process(&mut self) {
        while self.needs_work() {
            self.do_pass()
        }
    }
}

fn part2() -> usize {
    let mut card_set = CardSet {
        cards: INPUT.lines().map(Card::from).collect(),
        processed: Vec::new(),
        to_process: INPUT.lines().map(Card::from).collect(),
    };

    card_set.process();

    card_set.processed.len()
}
